/* -*- tab-width:8 c-basic-offset:4  indent-tabs-mode:nil -*- */
/* set vi: ts=8:softtabstop=4,shiftwidth=4,expandtab */

#ifndef NDS_OS_H
#define NDS_OS_H

#include <stdio.h>

#include "daqc.h"
#include "daqc_private.h"

/*
 * If you desire not to use the Windows _s functions, then add the following lines to the source file:
 *
 * #if _WIN32
 * #define _CRT_SECURE_NO_WARNINGS 1
 * #endif
 *
 */
void socket_io_non_blocking( nds_socket_type Socket );
void socket_io_default ( nds_socket_type Socket );

void pstrerror_r( char* StrErrBuf, size_t StrErrBufLen );

#if _WIN32
#include <io.h>

typedef size_t ssize_t;
typedef int socklen_t;

#ifdef __cplusplus

/* modes for access are not defined by io.h */
#define F_OK ((int)0)
#define W_OK ((int)2)

static inline int access( const char *path, int mode ) { return _access( path, mode ); }
static inline int close( nds_socket_type s ) { return closesocket( s ); }
static inline int dup( int fd ) { return _dup( fd ); }
static inline FILE *fdopen( int fd, const char *mode ) { return _fdopen( fd, mode ); }
static inline ssize_t read( nds_socket_type s, void *buf, size_t len) { return recv( s, reinterpret_cast<char *>(buf), static_cast<int>(len), 0 ); }
static inline void sleep(int seconds) { Sleep( seconds*1000 ); }
static inline char *strdup( const char *str ) { return _strdup( str ); }
static inline void usleep( int microseconds ) { Sleep( microseconds/1000 ); }
static inline ssize_t write( nds_socket_type s, const void *buf, size_t count) { return send( s, reinterpret_cast<const char *>(buf), count, 0); }
//static inline void perror( const char *msg ) { nds_perror( msg ); }
//static int unlink( const char *filename ) { return _unlink( filename ); }

static inline errno_t strerror_r( int errnum, char *buf, size_t buflen ) { return strerror_s( buf, buflen, errnum ); }

#define perror( message ) nds_perror( message )

#else

#define access(path,mode) _access(path,mode)
#define close(fd) closesocket(fd)
#define dup(oldfd) _dup(oldfd)
#define fdopen(path,mode) _fdopen(path,mode)
#define read(fd,buf,len) recv(fd,buf,len,0)
#define sleep(Seconds) Sleep(Seconds*1000)
#define strdup(str) _strdup(str)
#define usleep(MicroSeconds) Sleep(MicroSeconds/1000)
#define write(fd,buf,count) send(fd,buf,count,0)
#define perror( message ) nds_perror( message )
#define unlink(filename) _unlink(filename)

#define strerror_r(a,b,c)        strerror_s(b,c,a)
#endif /* __cplusplus */

#else /* _WIN32 */
#define strncpy_s(dest,dest_len,src,count) \
  strncpy(dest,src,count)
#define strcpy_s(dest,dest_len,src) \
  strcpy(dest,src)
#endif /* _WIN32 */

#endif /* NDS_OS_H */
